/**************************************************************************************************
	$Header: /pub/cvsroot/mydns/lib/mydnsutil.h,v 1.34 2002/09/20 21:15:29 bboy Exp $
	mydnsutil.h: Support library for the MyDNS package.

	Copyright (C) 2002  Don Moore <bboy@bboy.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at Your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************************************/

#ifndef MYDNSUTIL_H
#define MYDNSUTIL_H

#include <config.h>

/* <unistd.h> should be included before any preprocessor test
   of _POSIX_VERSION.  */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include <stdio.h>
#include <sys/types.h>

#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
#include <ctype.h>

#include <syslog.h>

#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif
#ifndef HAVE_SETLOCALE
#define setlocale(category,locale) /* empty */
#endif

/* For gettext (NLS).  */
#include "gettext.h"
#define _(String) gettext(String)
#define N_(String) (String)

#ifdef STDC_HEADERS
#define getopt system_getopt
#include <stdlib.h>
#undef getopt
#else
extern char *getenv ();
#endif

#include <getopt.h>

/* Don't use bcopy!  Use memmove if source and destination may overlap,
   memcpy otherwise.  */
#ifdef HAVE_STRING_H
# if !STDC_HEADERS && HAVE_MEMORY_H
#  include <memory.h>
# endif
# include <string.h>
#else
# include <strings.h>
char *memchr ();
#endif

#if HAVE_SIGNAL_H
#	include <signal.h>
#endif

#if HAVE_TERMIOS_H
#	include <termios.h>
#endif

#include <errno.h>
#ifndef errno
extern int errno;
#endif
#ifdef VMS
#include <perror.h>
#endif

#ifndef HAVE_DECL_STRERROR
extern char *strerror ();
#endif

#ifndef HAVE_DECL_STRCASECMP
extern int strcasecmp ();
#endif

#ifndef HAVE_DECL_STRNCASECMP
extern int strncasecmp ();
#endif

#ifndef HAVE_DECL_STRCOLL
extern int strcoll ();
#endif

#include <sys/stat.h>
#if STAT_MACROS_BROKEN
# undef S_ISDIR
#endif
#if !defined(S_ISDIR) && defined(S_IFDIR)
# define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
#endif

#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif /* HAVE_SYS_FILE_H */

#ifndef O_RDONLY
/* Since <fcntl.h> is POSIX, prefer that to <sys/fcntl.h>.
   This also avoids some useless warnings on (at least) Linux.  */
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#else /* not HAVE_FCNTL_H */
#ifdef HAVE_SYS_FCNTL_H
#include <sys/fcntl.h>
#endif /* not HAVE_SYS_FCNTL_H */
#endif /* not HAVE_FCNTL_H */
#endif /* not O_RDONLY */

#ifdef HAVE_SYS_SOCKET_H
#	include <sys/socket.h>
#endif

#ifdef HAVE_NETINET_IN_H
#	include <netinet/in.h>
#endif

#ifdef HAVE_ARPA_INET_H
#	include <arpa/inet.h>
#endif

#include <mysql.h>
#include <errmsg.h>

/* MS-DOS and similar non-Posix systems have some peculiarities:
    - they distinguish between binary and text files;
    - they use both `/' and `\\' as directory separator in file names;
    - they can have a drive letter X: prepended to a file name;
    - they have a separate root directory on each drive;
    - their filesystems are case-insensitive;
    - directories in environment variables (like INFOPATH) are separated
        by `;' rather than `:';
    - text files can have their lines ended either with \n or with \r\n pairs;

   These are all parameterized here except the last, which is
   handled by the source code as appropriate (mostly, in info/).  */
#ifndef O_BINARY
# ifdef _O_BINARY
#  define O_BINARY _O_BINARY
# else
#  define O_BINARY 0
# endif
#endif /* O_BINARY */

#if O_BINARY
# include <io.h>
# ifdef __MSDOS__
#  include <limits.h>
#  ifdef __DJGPP__
#   define HAVE_LONG_FILENAMES(dir)  (pathconf (dir, _PC_NAME_MAX) > 12)
#   define NULL_DEVICE	"/dev/null"
#  else  /* !__DJGPP__ */
#   define HAVE_LONG_FILENAMES(dir)  (0)
#   define NULL_DEVICE	"NUL"
#  endif /* !__DJGPP__ */
#  define SET_SCREEN_SIZE_HELPER terminal_prep_terminal()
#  define DEFAULT_INFO_PRINT_COMMAND ">PRN"
# else   /* !__MSDOS__ */
#  define setmode(f,m)  _setmode(f,m)
#  define HAVE_LONG_FILENAMES(dir)   (1)
#  define NULL_DEVICE	"NUL"
# endif  /* !__MSDOS__ */
# define SET_BINARY(f)  do {if (!isatty(f)) setmode(f,O_BINARY);} while(0)
# define FOPEN_RBIN	"rb"
# define FOPEN_WBIN	"wb"
# define IS_SLASH(c)	((c) == '/' || (c) == '\\')
# define HAVE_DRIVE(n)	((n)[0] && (n)[1] == ':')
# define IS_ABSOLUTE(n)	(IS_SLASH((n)[0]) || ((n)[0] && (n)[1] == ':'))
# define FILENAME_CMP	strcasecmp
# define FILENAME_CMPN	strncasecmp
# define PATH_SEP	";"
# define STRIP_DOT_EXE	1
# define DEFAULT_TMPDIR	"c:/"
# define PIPE_USE_FORK	0
#else  /* not O_BINARY */
# define SET_BINARY(f)	(void)0
# define FOPEN_RBIN	"r"
# define FOPEN_WBIN	"w"
# define IS_SLASH(c)	((c) == '/')
# define HAVE_DRIVE(n)	(0)
# define IS_ABSOLUTE(n)	((n)[0] == '/')
# define FILENAME_CMP	strcmp
# define FILENAME_CMPN	strncmp
# define HAVE_LONG_FILENAMES(dir)   (1)
# define PATH_SEP	":"
# define STRIP_DOT_EXE	0
# ifdef VMS
#  define DEFAULT_TMPDIR "sys$scratch:"
# else
#  define DEFAULT_TMPDIR "/tmp/"
# endif
# define NULL_DEVICE	"/dev/null"
# define PIPE_USE_FORK	1
#endif /* not O_BINARY */

#if HAVE_TIME_H
#	include <time.h>
#endif
#if HAVE_SYS_TIME_H
#	include <sys/time.h>
#endif

#if HAVE_PWD_H
#	include <pwd.h>
#endif


/* Placing this after a function prototype indicates to the compiler that the function takes a
   variable number of arguments in printf(3) format.  If this is used, the compiler will check
   to make sure your variables match the format string, just like it would with printf(3) and
   friends.  You shouldn't use this if it's possible to pass NULL as the fmtarg, since the
   the compiler will issue a warning about a NULL format string. */
/* GNU only... but this is for Linux iptables only so does it really matter? */
#define  __printflike(fmtarg, firstvararg) \
            __attribute__((__format__ (__printf__, fmtarg, firstvararg)))

/* Macro for pluralizing words in printf() statements.  If 'n' is 1, evaluates to "", otherwise
   evaluates to "s".  i.e. printf("%d number%s found.", num, S(num)); */
#define  S(n)  ((n == 1) ? "" : "s")

/* Evaluates to the number of items in the specified array. */
#define  NUM_ENTRIES(Array) (sizeof Array / sizeof *(Array))

/* Macro to return the last char in a string */
#define	LASTCHAR(s)	(s[strlen(s)-1])


/* Macro to determine an average.  "t" is the total, and "c" is the number of elements. */
#define  AVG(t,c)    (double)(((double)c > 0.0) ? (double)((double)t / (double)c) : 0.0)

/* Macro to determine a percentage.  "t" is the total, "c" is the count. i.e. PCT(100,65) 
	evaluates as (double)65.0. */
#define  PCT(t,c)    (double)(((double)t > 0.0) ? (double)((double)c / (double)t) * 100.0 : 0.0)


/*
**  conf.c
**  Routines to load the configuration file
*/
/* Generic structure for holding name/value pairs */
typedef struct _conflist
{
	unsigned char *name;			/* Name of option */
	unsigned char *value;		/* Value for this option */
	unsigned char *desc;			/* Description of this option */
	unsigned char *altname;		/* Alternate name for this option */
	struct _conflist *next;
} CONF;

#define	CONF_FS_CHAR	'\034'
#define	CONF_FS_STR		"\034"
extern void conf_set(CONF **, unsigned char *, unsigned char *);
extern unsigned char *conf_get(CONF **, unsigned char *);
extern void conf_load(CONF **, const unsigned char *);


/*
**  error.c
**  Error reporting functions.
*/
extern char *progname;											/* The name of this program */
extern int	err_verbose;										/* Should ERR_VERBOSE output anything? */
extern int	err_debug;											/* Should ERR_DEBUG output anything? */
extern int	err_plain;											/* Simple, unadorned error messages? */
extern int	err_stderr;											/* Output to stderr only (no syslog)? */


extern void error_init(const char *argv0, int facility);
extern void Debug(const char *, ...) __printflike(1,2);
extern void Verbose(const char *, ...) __printflike(1,2);
extern void Notice(const char *, ...) __printflike(1,2);
extern int	Warn(const char *, ...) __printflike(1,2);
extern int	Warnx(const char *, ...) __printflike(1,2);
extern void Err(const char *, ...) __printflike(1,2);
extern void Errx(const char *, ...) __printflike(1,2);
extern int	WarnSQL(MYSQL *, const char *, ...) __printflike(2,3);
extern void ErrSQL(MYSQL *, const char *, ...) __printflike(2,3);

extern void _error_assert_fail();							/* Assertion failure function */

/* Error reporting and output macros */
#define Assert(expr)				((void)((expr) || (_error_assert_fail(__STRING(expr)), 0)))

#ifdef notdef
/*
**  numlist.c
**  Simple list of numbers.
*/
typedef struct _numlist
{
	size_t		size;
	u_int64_t	*num;
} NUMLIST;
extern NUMLIST *numlist_insert(NUMLIST *, uint64_t);
extern void _numlist_free(NUMLIST **);
#define	numlist_free(l)	_numlist_free(&(l))
extern int numlist_exists(NUMLIST *, u_int64_t);
#endif

#ifndef HAVE_INET_PTON
extern int inet_pton(int, const char *, void *);
#endif

#ifndef HAVE_INET_NTOP
extern const char *inet_ntop(int, const void *, char *, unsigned int);
#endif


/*
**  Quickies
*/
extern char *getoptstr(struct option const longopts[]);									/* getoptstr.c */
extern char *passinput(const char *prompt);													/* passinput.c */
extern char *strtrimlead(char *), *strtrimtrail(char *), *strtrim(char *);			/* string.c */
extern char *strtoupper(char *), *strtolower(char *);										/* string.c */
extern char *strsecs(time_t);																		/* string.c */
extern char *strdcat(char **, const char *);													/* string.c */
extern int	sdprintf(char **, const char *, ...) __printflike(2,3);					/* string.c */
extern size_t human_file_size(const char *);													/* string.c */
extern void escdata(unsigned char *, int);													/* string.c */
extern char *bytestr(unsigned char);															/* string.c */
extern char *escstr();
extern void *xmalloc(int), *xcalloc(int, int), *xrealloc(void *, int);				/* xmalloc.c */
extern unsigned char *xstrdup(unsigned char *);												/* xstrdup.c */
extern int wildcard_valid(char *p);																/* wildcard.c */
extern inline int wildcard_match(register char *p, register char *);					/* wildcard.c */

#endif /* MYDNSUTIL_H */

/* vi:set ts=3: */
